/*
 * Copyright (c) 2015, Freescale Semiconductor, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of Freescale Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Modifications Copyright (c) 2016 ARM Germany GmbH. All rights reserved.
 */

#include <stdlib.h>
#include <string.h>
#include "rpmsg.h"
#include "rpmsg_ext.h"
#include "rpmsg_rtos.h"
#include "board.h"
#include "mu_imx.h"
#include "adc_imx7d.h"
#include "cmsis_os.h"                   // ARM::CMSIS:RTOS:Keil RTX
#include "retarget_io_user.h"
#include "arm_math.h"
#include "arm_const_structs.h"

#define PRINTF printf

#define LENGTH_SAMPLES 2048

/*
 * APP decided interrupt priority
 */
#define APP_MU_IRQ_PRIORITY 3

/* Globals */
char app_buf[512]; /* Each RPMSG buffer can carry less than 512 payload */

volatile uint32_t freq;
uint32_t samples = 0;

// DSP Lib
__attribute__ ((at(0x9FF20000))) float32_t input[LENGTH_SAMPLES*2];
__attribute__ ((at(0x9FF20000))) float32_t output[LENGTH_SAMPLES];
uint32_t i = 0;
uint32_t maxIndex = 0;
float32_t maxValue;


////////////////////////////////////////////////////////////////////////////////
// Definitions
////////////////////////////////////////////////////////////////////////////////
/*
 * APP decided interrupt priority
 */
#define APP_MU_IRQ_PRIORITY 3

void TTYThread (void const *arg);
osThreadDef (TTYThread, osPriorityNormal, 1, 0);

void ADCThread (void const *arg);
osThreadDef (ADCThread, osPriorityNormal, 1, 0);

typedef struct the_message
{
  uint32_t  DATA;
} THE_MESSAGE, * THE_MESSAGE_PTR;

////////////////////////////////////////////////////////////////////////////////
// Code
////////////////////////////////////////////////////////////////////////////////

void TTYThread (void const *arg)
{
  int result;
  struct remote_device *rdev = NULL;
  struct rpmsg_channel *app_chnl = NULL;
  void *rx_buf;
  int len;
  unsigned long src;
  void *tx_buf;
  unsigned long size;
  char str[16];

  /* Print the initial banner */
  PRINTF("\r\nFrequency bin application\r\n");

  /* RPMSG Init as REMOTE */
  PRINTF("RPMSG Init as Remote\r\n");
  result = rpmsg_rtos_init(0 /*REMOTE_CPU_ID*/, &rdev, RPMSG_MASTER, &app_chnl);
  assert(result == 0);

  /* reinitialize debug uart */
  InitRetargetIOUSART();

  PRINTF("Name service handshake is done, M4 has setup a rpmsg channel [%ld ---> %ld]\r\n", app_chnl->src, app_chnl->dst);

  while (true)
  {
    /* Get RPMsg rx buffer with message */
    result = rpmsg_rtos_recv_nocopy(app_chnl->rp_ept, &rx_buf, &len, &src, 0xFFFFFFFF);
    assert(result == 0);

    /* Copy string from RPMsg rx buffer */
    assert(len < sizeof(app_buf));
    memcpy(app_buf, rx_buf, len);
    app_buf[len] = 0; /* End string by '\0' */

    /* If any message has arrived, respond with last calculated frequency bin */
    if (len>0) {
      tx_buf = rpmsg_rtos_alloc_tx_buffer(app_chnl->rp_ept, &size);
      assert(tx_buf);

      sprintf(str,"%4dHz",freq);
      memcpy(tx_buf, str, 6);

      /* Send message with nocopy */
      result = rpmsg_rtos_send_nocopy(app_chnl->rp_ept, tx_buf, 6, src);
      assert(result == 0);
    }

    /* Release held RPMsg rx buffer */
    result = rpmsg_rtos_recv_nocopy_free(app_chnl->rp_ept, rx_buf);
    assert(result == 0);
  }

}

void ADCThread (void const *arg)
{
  adc_init_config_t adcConfig = {
      .sampleRate = 1000000,
      // For i.MX7D this value should always be set to true.
      .levelShifterEnable = true
  };

  adc_logic_ch_init_config_t adcChannelConfig = {
      .inputChannel = BOARD_ADC_INPUT_CHANNEL,
      .coutinuousEnable = true,
      .convertRate = LENGTH_SAMPLES,
      .averageEnable = true,
      .averageNumber = adcAvgNum32
  };

  // Initialize ADC module.
  ADC_Init(BOARD_ADC_BASEADDR, &adcConfig);
  // Enable Convert finish interrupt on Logic Channel A
  ADC_SetIntSigCmd(BOARD_ADC_BASEADDR, adcIntConvertChA, true);
  ADC_SetIntCmd(BOARD_ADC_BASEADDR, adcIntConvertChA, true);

  /* Set ADC Interrupt priority */
  NVIC_SetPriority(BOARD_ADC_IRQ_NUM, 3);
  /* Call core API to enable the IRQ. */
  NVIC_EnableIRQ(BOARD_ADC_IRQ_NUM);

  // Initialize ADC Logic Channel A module.
  ADC_LogicChInit(BOARD_ADC_BASEADDR, adcLogicChA, &adcChannelConfig);

  while (true)
  {
    samples = 0;
    i = 0;
    memset(input,0,LENGTH_SAMPLES*2*sizeof(float32_t));

    ADC_SetConvertCmd(BOARD_ADC_BASEADDR, adcLogicChA, true);

    osDelay(1000);

    ADC_SetConvertCmd(BOARD_ADC_BASEADDR, adcLogicChA, false);

    /* Process the data through the CFFT/CIFFT module */
    arm_cfft_f32(&arm_cfft_sR_f32_len2048, input, 0, 1);

    /* Process the data through the Complex Magnitude Module for
       calculating the magnitude at each bin */
    arm_cmplx_mag_f32(input, output, LENGTH_SAMPLES/2);

    /* Remove DC component */
    output[0]=0;
    output[1]=0;

    /* Calculates maxValue and returns corresponding BIN value up to half of sample rate */
    arm_max_f32(output, LENGTH_SAMPLES/2, &maxValue, &maxIndex);

    /* freq = samples per second / number of samples * index of
       max corresponding BIN value */
    freq = roundf(((float)samples*maxIndex)/LENGTH_SAMPLES);
    PRINTF("%dS/s %dHz \r\n", samples, freq);
  }
}

int main(void)
{
  hardware_init();
  
  SystemCoreClockUpdate();
  
  InitRetargetIOUSART();

  /* Initialize CMSIS-RTOS */
  osKernelInitialize(); 
  
  /* Create new threads */
  osThreadCreate (osThread (TTYThread), NULL);
  osThreadCreate (osThread (ADCThread), NULL); 
  
  /* Start thread execution */ 
  osKernelStart();                    

  /* Infinite loop */
  while(true);
}

void BOARD_ADC_HANDLER(void)
{
  ADC_ClearStatusFlag(BOARD_ADC_BASEADDR, adcStatusConvertChA);
  uint16_t res = ADC_GetConvertResult(BOARD_ADC_BASEADDR, adcLogicChA);
  samples++;

  if (i<(LENGTH_SAMPLES*2)) {
    input[i++] = res;
    input[i++] = 0;
  }
}

/*******************************************************************************
 * EOF
 ******************************************************************************/
