/*
 * Copyright (c) 2015, Freescale Semiconductor, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of Freescale Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * Modifications Copyright (c) 2016 ARM Germany GmbH. All rights reserved.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "rpmsg.h"
#include "rpmsg_ext.h"
#include "rpmsg_rtos.h"
#include "board.h"
#include "mu_imx.h"
#include "cmsis_os.h"                   // ARM::CMSIS:RTOS:Keil RTX
#include "rl_net.h"                     /* Network definitions                */

#define TCP_PORT  1000U
#define UDP_PORT  6000U

// Thread IDs
osThreadId TID_DebugViewer;
osThreadId TID_SocketHandle;

// Thread definitions
static void DebugViewer  (void const *arg);
osThreadDef(DebugViewer,  osPriorityNormal, 1, 0);
static void SocketHandle (void const *arg);
osThreadDef(SocketHandle,  osPriorityNormal, 1, 0);

// TCP definitions
int32_t TCP_socket;
uint8_t *TCP_buf;
uint32_t TCP_len = 0;

// UDP definitions
int32_t UDP_socket;
NET_ADDR UDP_addr;
uint8_t *UDP_buf;
uint32_t UDP_len = 0;

/// IP address change notification
void netDHCP_Notify (uint32_t if_num, uint8_t option, const uint8_t *val, uint32_t len) {
  if (option == NET_DHCP_OPTION_IP_ADDRESS) {
    // IP address has changed
	osSignalSet (TID_DebugViewer, 0x01);
  }
}

/*----------------------------------------------------------------------------
  Thread 'DebugViewer'
 *---------------------------------------------------------------------------*/
static void DebugViewer (void const *arg) {
  static uint8_t ip_addr[NET_ADDR_IP6_LEN];
  static char    ip_ascii[40];

  // Print Link-local IPv6 address
  netIF_GetOption (NET_IF_CLASS_ETH,
                   netIF_OptionIP6_LinkLocalAddress, ip_addr, sizeof(ip_addr));

  netIP_ntoa(NET_ADDR_IP6, ip_addr, ip_ascii, sizeof(ip_ascii));
  printf ("IP6:%.16s%s\n", ip_ascii, ip_ascii+16);

  while(1) {
    /* Wait for signal from DHCP */
    osSignalWait (0x01, osWaitForever);
    /* Retreive and print IPv4 address */
    netIF_GetOption (NET_IF_CLASS_ETH,
                     netIF_OptionIP4_Address, ip_addr, sizeof(ip_addr));

    netIP_ntoa (NET_ADDR_IP4, ip_addr, ip_ascii, sizeof(ip_ascii));
    printf ("IP4:%-16s\n",ip_ascii);
  }
}

/*------------------------------------------------------------------------------
  TCP_callback
 *------------------------------------------------------------------------------*/
uint32_t TCP_callback (int32_t soc, netTCP_Event evt, const NET_ADDR *addr, const uint8_t *ptr, uint32_t par) {

  switch (evt) {
    case netTCP_EventConnect:
      /* Remote host is trying to connect to our TCP socket. */
      /* 'ptr' points to Remote IP, 'par' holds the remote port. */
      return (1);                              // Return 1 to accept connection
    case netTCP_EventAborted:
      /* Connection was aborted */
      break;
    case netTCP_EventEstablished:
      /* Socket is connected to remote peer. */
      break;
    case netTCP_EventClosed:
      /* Connection has been closed */
      break;
    case netTCP_EventACK:
      /* Our sent data has been acknowledged by remote peer */
      break;
    case netTCP_EventData:
      /* TCP data frame has been received, 'ptr' points to data */
      /* Data length is 'par' bytes */
      TCP_buf = netTCP_GetBuffer(par);
      memcpy(TCP_buf, ptr, par);
      TCP_len = par;
      break;
    default:
      break ;
  }

  return (0);
}

/*------------------------------------------------------------------------------
  UDP_callback
 *------------------------------------------------------------------------------*/
uint32_t UDP_callback (int32_t socket, const NET_ADDR *addr, const uint8_t *buf, uint32_t len) {
  // This function is called when UDP data is received
  // Process received data from 'buf'
  UDP_buf = netUDP_GetBuffer(len);
  memcpy(UDP_buf, buf, len);
  memcpy(&UDP_addr, addr, sizeof(NET_ADDR));
  UDP_len = len;
  return (0);
}

/*----------------------------------------------------------------------------
  Thread 'SocketHandle'
 *---------------------------------------------------------------------------*/
static void SocketHandle (void const *arg) {

  TCP_socket = netTCP_GetSocket (TCP_callback);
  netTCP_Listen (TCP_socket, TCP_PORT);
  UDP_socket = netUDP_GetSocket (UDP_callback);
  netUDP_Open (UDP_socket, UDP_PORT);

  while(1) {
    if (TCP_len>0) {
      printf("TCP: %.*s\n", TCP_len, TCP_buf);
      netTCP_Send(TCP_socket, TCP_buf, TCP_len);
      TCP_len = 0;
    }
    if (UDP_len>0) {
      printf("UDP: %.*s\n", UDP_len, UDP_buf);
      netUDP_Send(UDP_socket, (NET_ADDR *)&UDP_addr, UDP_buf, UDP_len);
      UDP_len = 0;
    }
  }
}

/*----------------------------------------------------------------------------
  Main Thread 'main': Run Network
 *---------------------------------------------------------------------------*/
int main (void) {

  hardware_init();

  // System Initialization
  SystemCoreClockUpdate();

  /* Network initialization */
  netInitialize();

  TID_DebugViewer = osThreadCreate (osThread(DebugViewer), NULL);
  TID_SocketHandle = osThreadCreate (osThread(SocketHandle), NULL);

  while(1);
}

